import { declare } from '@babel/helper-plugin-utils';
import { types, template } from '@babel/core';

const TRACE_ID = "__source";
const FILE_NAME_VAR = "_jsxFileName";
const createNodeFromNullish = (val, fn) => val == null ? types.nullLiteral() : fn(val);
var index = declare(api => {
  api.assertVersion(7);
  function makeTrace(fileNameIdentifier, {
    line,
    column
  }) {
    const fileLineLiteral = createNodeFromNullish(line, types.numericLiteral);
    const fileColumnLiteral = createNodeFromNullish(column, c => types.numericLiteral(c + 1));
    return template.expression.ast`{
      fileName: ${fileNameIdentifier},
      lineNumber: ${fileLineLiteral},
      columnNumber: ${fileColumnLiteral},
    }`;
  }
  const isSourceAttr = attr => types.isJSXAttribute(attr) && attr.name.name === TRACE_ID;
  return {
    name: "transform-react-jsx-source",
    visitor: {
      JSXOpeningElement(path, state) {
        const {
          node
        } = path;
        if (!node.loc || path.node.attributes.some(isSourceAttr)) {
          return;
        }
        if (!state.fileNameIdentifier) {
          const fileNameId = path.scope.generateUidIdentifier(FILE_NAME_VAR);
          state.fileNameIdentifier = fileNameId;
          path.scope.getProgramParent().push({
            id: fileNameId,
            init: types.stringLiteral(state.filename || "")
          });
        }
        node.attributes.push(types.jsxAttribute(types.jsxIdentifier(TRACE_ID), types.jsxExpressionContainer(makeTrace(types.cloneNode(state.fileNameIdentifier), node.loc.start))));
      }
    }
  };
});

export { index as default };
//# sourceMappingURL=index.js.map
